package lucastest;


public class CalculatorForPosterior {

    private ProbabilitiesForACoalescence[] coalescenceProbs;

    private double logPrior; // for text output




    CalculatorForPosterior() {

    }



    public double logPosterior(SpeciesSet speciesSet, GTree gTree) {
        findAllCoalescenceProbabilities(speciesSet, gTree);
        double logP = 0;
        for (int n = 0; n < coalescenceProbs.length; n++) {
            logP += logPForOneCoalescence(coalescenceProbs[n]);
        }
        assert Double.isFinite(logP);
        logPrior = logP;
        return logP;
    }



    public String asText() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("\nAll coalescenceProbs\n");
        for (int n = 0; n < coalescenceProbs.length; n++) {
            ProbabilitiesForACoalescence coalPrs = coalescenceProbs[n];
            buffer.append("\ncoalescenceProbs for node ");
            buffer.append(coalPrs.asText());
            buffer.append("\n");
        }

        buffer.append(String.format("######################\nlogPrior %8.6f  prior %8.6f \n####################\n", logPrior, Math.exp(logPrior)));
        return buffer.toString();
    }



    /*********************************************************************************************************************/
    /*********************************************************************************************************************/
    /*********************************************************************************************************************/

    private double logPForOneCoalescence(ProbabilitiesForACoalescence coalPrs) {
        double logP = 0.0;
        ProbabilitiesForCoalescers pfc = coalPrs.coalescerPrs;
        double PrTijAiAj =
                pfc.PrTijGivenAiAjN00 * pfc.PrAiNi0 * pfc.PrAjNj0 +
                        pfc.PrTijGivenAiAjN01 * pfc.PrAiNi0 * pfc.PrAjNj1 +
                        pfc.PrTijGivenAiAjN10 * pfc.PrAiNi1 * pfc.PrAjNj0 +
                        pfc.PrTijAiAjN2plus;
        logP += Math.log(PrTijAiAj);
        assert Double.isFinite(logP);
        logP += Math.log(pfc.cRate);
        for (int k = 0; k < coalPrs.persisterPrs.length; k++) {
            ProbabilitiesForPersister pfr = coalPrs.persisterPrs[k];
            double PrTijkAiAj = 0;
            PrTijkAiAj += pfr.PrTijkGivenAiAjN000 * pfc.PrAiNi0 * pfc.PrAjNj0 * pfr.PrNk0;
            PrTijkAiAj += pfr.PrTijkGivenAiAjN001 * pfc.PrAiNi0 * pfc.PrAjNj0 * pfr.PrNk1;
            PrTijkAiAj += pfr.PrTijkGivenAiAjN010 * pfc.PrAiNi0 * pfc.PrAjNj1 * pfr.PrNk0;
            PrTijkAiAj += pfr.PrTijkGivenAiAjN100 * pfc.PrAiNi1 * pfc.PrAjNj0 * pfr.PrNk0;
            PrTijkAiAj += pfr.PrTijkGivenAiAjN110 * pfc.PrAiNi1 * pfc.PrAjNj1 * pfr.PrNk0;
            PrTijkAiAj += pfr.PrTijkGivenAiAjNik2plusNjk2plus * pfc.PrAi * pfc.PrAj * pfr.PrNik2plusNjk2plus;
            double PrTijkGivenAiAj = PrTijkAiAj / (pfc.PrAi * pfc.PrAj);
            logP += Math.log(PrTijkGivenAiAj);
            assert Double.isFinite(logP);
            assert Double.isFinite(logP);
        }
        return logP;
    }


    private void findAllCoalescenceProbabilities(SpeciesSet speciesSet, GTree gTree) {
        int [] coalsInHeightOrder = gTree.coalsInHeightOrder();
        int nCoals = coalsInHeightOrder.length;
        assert nCoals == gTree.getNTips() - 1;
        coalescenceProbs = new ProbabilitiesForACoalescence[nCoals];
        for (int idx = 0; idx < nCoals; idx++) {
            int n = coalsInHeightOrder[idx];
            coalescenceProbs[idx] = new ProbabilitiesForACoalescence(speciesSet, gTree, n);
        }
    }

}
