package lucastest;


import java.io.FileWriter;
import java.io.IOException;

public class LTMain {

    static String outDirectoryRoot = "C:/Users/GRJ/AAA/ProgramOutput/Lucas";



    static void makeDirectSampleOfGtrees(String fpath, Config config, SpeciesSet speciesSet, int nSmps) {
        DirectSample directSample = new DirectSample();
        directSample.makeCollection(config, speciesSet, fpath, nSmps);
    }


    static void makeMCMCSample(String fpath, Config config, SpeciesSet speciesSet, int nSmps) {
        MCMC mcmc = new MCMC(config, speciesSet, (nSmps-1)*1000 + 1, 1000);
        GTree gTree = new GTree(config, speciesSet, false);
        gTree.makeSampleTree();
        mcmc.run(gTree, fpath);
    }


    static void makePosteriorForOneGtree(String fpath, Config config) {
        SpeciesSet speciesSet = new SpeciesSet(config);
        GTree gTree = new GTree(config, speciesSet, false);
        gTree.makeSampleTree();
        CalculatorForPosterior posterior = new CalculatorForPosterior();
        posterior.logPosterior(speciesSet, gTree);

        try {
            FileWriter fw = new FileWriter(fpath);
            fw.write(config.asText());
            fw.write(gTree.asText());
            fw.write(posterior.asText());
            fw.write("\n");
            fw.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    static void doPosteriorDirectMCMC(String leaf) {
        String dpath = outDirectoryRoot + "/" + leaf;
        Config config = new Config(dpath + "/config.txt");
        System.out.println("\n" + config.asText());
        makePosteriorForOneGtree( dpath + "/posterior.txt", config);
        int nSmps = 1000;
        SpeciesSet speciesSet = new SpeciesSet(config);
        makeDirectSampleOfGtrees(dpath + "/directCollection.txt", config, speciesSet, nSmps);
        makeMCMCSample(dpath + "/MCMC.txt", config, speciesSet, nSmps);
        System.out.println("Output in: " + dpath);
    }


    private static void do2lintests() {
        doPosteriorDirectMCMC("Testspp2lins2m05");
        doPosteriorDirectMCMC("Testspp2lins2m5");
        doPosteriorDirectMCMC("Testspp2lins2m50");
        doPosteriorDirectMCMC("Testspp2lins2m500");

        doPosteriorDirectMCMC("Testspp3lins2m05");
        doPosteriorDirectMCMC("Testspp3lins2m5");
        doPosteriorDirectMCMC("Testspp3lins2m50");
        doPosteriorDirectMCMC("Testspp3lins2m500");

        doPosteriorDirectMCMC("Testspp6lins2m05");
        doPosteriorDirectMCMC("Testspp6lins2m5");
        doPosteriorDirectMCMC("Testspp6lins2m50");
        doPosteriorDirectMCMC("Testspp6lins2m500");
    }


    private static void do5spp5lintests() {
        doPosteriorDirectMCMC("Testspp5lins1+1+1+1+1m05");
        doPosteriorDirectMCMC("Testspp5lins1+1+1+1+1m5");
        doPosteriorDirectMCMC("Testspp5lins1+1+1+1+1m50");
        doPosteriorDirectMCMC("Testspp5lins1+1+1+1+1m500");
    }


    private static void do5spp20lintests() {
        doPosteriorDirectMCMC("Testspp5lins4+4+4+4+4m05");
        doPosteriorDirectMCMC("Testspp5lins4+4+4+4+4m5");
        doPosteriorDirectMCMC("Testspp5lins4+4+4+4+4m50");
        doPosteriorDirectMCMC("Testspp5lins4+4+4+4+4m500");
    }


    private static void do3spp5lintests() {
        doPosteriorDirectMCMC("Testspp3lins3+2+0m05");
        doPosteriorDirectMCMC("Testspp3lins3+2+0m5");
        doPosteriorDirectMCMC("Testspp3lins3+2+0m50");
        doPosteriorDirectMCMC("Testspp3lins3+2+0m500");
    }



    public static void main(String[] args) {
        do2lintests();
        do5spp5lintests();
        do3spp5lintests();
        do5spp20lintests();
    }
}
