package lucastest;

public class ProbabilitiesForCoalescers {
    SpeciesSet speciesSet;
    GTree gTree;
    int n;

    double PrNi0;
    double PrNj0;
    double PrNi1;
    double PrNj1;
    double PrN2plus;
    double PrAi;
    double PrAj;
    double PrAiNi0;
    double PrAjNj0;
    double PrAiNi1;
    double PrAjNj1;
    double PrTijGivenAiAjN00;
    double PrTijGivenAiAjN01;
    double PrTijGivenAiAjN10;
    double PrTijAiAjN2plus;
    double cRate;

    ProbabilitiesForCoalescers(SpeciesSet speciesSet, GTree gTree, int n) {
        this.speciesSet = speciesSet;
        this.gTree = gTree;
        this.n = n;
        GNode node = gTree.getNode(n);
        int i = node.getChX();
        int j = node.getChY();
        Lineage iLin = new Lineage(gTree, i);
        Lineage jLin = new Lineage(gTree, j);
        cRate = speciesSet.getCRate(iLin.endB);
        double mRate = speciesSet.getMRate();
        double s = speciesSet.getNTips();
        MigrationModel migrationModel = new MigrationModel(s, mRate);

        double ui = Math.max(jLin.startT - iLin.startT, 0.0);
        double uj = Math.max(iLin.startT - jLin.startT, 0.0);
        double w = iLin.endT - Math.max(iLin.startT, jLin.startT);
        double iMT = (s-1) * mRate * (ui + w);
        PrNi0 = Math.exp(-iMT);
        PrNi1 = iMT * Math.exp(-iMT);
        double jMT = (s-1) * mRate * (uj + w);
        PrNj0 = Math.exp(-jMT);
        PrNj1 = jMT * Math.exp(-jMT);
        PrN2plus = 1.0 - PrNi0*PrNj0 - PrNi0*PrNj1 - PrNi1*PrNj0;
        int iSbIsEb = (iLin.startB == iLin.endB) ? 1 : 0;
        int jSbIsEb = (jLin.startB == jLin.endB) ? 1 : 0;
        int iSbIsNotEb = 1 - iSbIsEb;
        int jSbIsNotEb = 1 - jSbIsEb;
        PrAiNi0 = iSbIsEb * PrNi0;
        PrAjNj0 = jSbIsEb * PrNj0;
        PrAiNi1 = (1 - iSbIsEb) * PrNi1 / (s-1.0);
        PrAjNj1 = (1 - jSbIsEb) * PrNj1 / (s-1.0);
        double ecw = Math.exp(-cRate*w);
        PrTijGivenAiAjN00 = iSbIsEb * jSbIsEb * ecw;
        PrTijGivenAiAjN01 = iSbIsEb * jSbIsNotEb * ((uj/(uj+w)) * ecw + (w/(uj+w)) * Utils.oneMinusExpMinusXOverX(cRate*w));
        PrTijGivenAiAjN10 = iSbIsNotEb * jSbIsEb * ((ui/(ui+w)) * ecw + (w/(ui+w)) * Utils.oneMinusExpMinusXOverX(cRate*w));
        PrAi = migrationModel.getPrInBranch(iSbIsEb, ui+w);
        PrAj = migrationModel.getPrInBranch(jSbIsEb, uj+w);

        double PrStaysi = migrationModel.getPrInBranch(1, ui+w);
        double PrStaysj = migrationModel.getPrInBranch(1, uj+w);
        double PrMovesi = migrationModel.getPrInBranch(0, ui+w);
        double PrMovesj = migrationModel.getPrInBranch(0, uj+w);
        double prTog = 0;
        int iSbIsjSb = 0;
        if (iLin.startB == jLin.startB) {
            prTog = PrStaysi*PrStaysj + (s-1) * PrMovesi * PrMovesj;
            iSbIsjSb = 1;
        } else {
            prTog = PrStaysi*PrMovesj + PrMovesi*PrStaysj + (s-2) * PrMovesi * PrMovesj;
        }
        double PrAiAjGivenTog = PrAi * PrAj / prTog;
        ThreeStateCTAModel threeStateCTAModel = new ThreeStateCTAModel(s, mRate, speciesSet.getMeanCRate());
        double CTAPr2WhenIandJStarted = migrationModel.getPrInBranch(iSbIsjSb, ui+uj);
        double CTAPr2AtCoal = threeStateCTAModel.getCTAPr2Given3(w) * (1-CTAPr2WhenIandJStarted) + threeStateCTAModel.getCTAPr2Given2(w)*CTAPr2WhenIandJStarted;
        PrTijAiAjN2plus = PrN2plus * CTAPr2AtCoal * PrAiAjGivenTog;
        if (!Utils.validProbability(PrTijAiAjN2plus)) {
            System.out.println("DEBUGGING");
        }
        checkValidity();
    }


    private void checkValidity() {
        assert Double.isFinite(cRate);
        assert Utils.validProbability(PrNi0);
        assert Utils.validProbability(PrNi1);
        assert Utils.validProbability(PrNj0);
        assert Utils.validProbability(PrNj1);
        assert Utils.validProbability(PrN2plus);
        assert Utils.validProbability(PrAiNi0);
        assert Utils.validProbability(PrAjNj0);
        assert Utils.validProbability(PrAiNi1);
        assert Utils.validProbability(PrAjNj1);
        assert Utils.validProbability(PrTijGivenAiAjN00);
        assert Utils.validProbability(PrTijGivenAiAjN01);
        assert Utils.validProbability(PrTijGivenAiAjN10);
        assert Utils.validProbability(PrAi);
        assert Utils.validProbability(PrAj);
        assert Utils.validProbability(PrTijAiAjN2plus);
    }

        public String asText() {
        StringBuffer buffer = new StringBuffer();
        buffer.append(String.format("node index %d\n", n));
        buffer.append(String.format("PrNi0 %s PrNj0 %s ", Utils.nonNegIn8Chars(PrNi0), Utils.nonNegIn8Chars(PrNj0)));
        buffer.append(String.format("PrNi1 %s PrNj1 %s ", Utils.nonNegIn8Chars(PrNi1), Utils.nonNegIn8Chars(PrNj1)));
        buffer.append(String.format("PrN2plus %s\n", Utils.nonNegIn8Chars(PrN2plus)));
        buffer.append(String.format("PrAi %s PrAj %s ", Utils.nonNegIn8Chars(PrAi), Utils.nonNegIn8Chars(PrAj)));
        buffer.append(String.format("PrAiNi0 %s PrAjNj0 %s ", Utils.nonNegIn8Chars(PrAiNi0), Utils.nonNegIn8Chars(PrAjNj0)));
        buffer.append(String.format("PrAiNi1 %s PrAjNj1 %s\n", Utils.nonNegIn8Chars(PrAiNi1), Utils.nonNegIn8Chars(PrAjNj1)));
        buffer.append(String.format("PrTijGivenAiAjN00 %s PrTijGivenAiAjN01 %s PrTijGivenAiAjN10 %s ",
                Utils.nonNegIn8Chars(PrTijGivenAiAjN00), Utils.nonNegIn8Chars(PrTijGivenAiAjN01), Utils.nonNegIn8Chars(PrTijGivenAiAjN10)));
        buffer.append(String.format("PrTijAiAjN2plus %s\n", Utils.nonNegIn8Chars(PrTijAiAjN2plus)));

        return buffer.toString();

    }

}
